@@ -1,37 +1,51 @@
 #!/usr/bin/perl
-# $Id: podbrowser.pl,v 1.20 2004/09/18 13:32:10 jodrell Exp $
+# $Id: podbrowser.pl,v 1.43 2006/08/08 10:52:40 gavin Exp $
 # Copyright (c) 2004 Gavin Brown. All rights reserved. This program is free
 # software; you can redistribute it and/or modify it under the same terms
 # as Perl itself.
+use Data::Dumper;
+use File::Basename qw(dirname);
+use File::Temp qw(tempdir tempfile);
 use Gtk2 -init;
 use Gtk2::GladeXML 1.001;
-use Gtk2::SimpleList;
-use Gtk2::PodViewer 0.08;
-use Gtk2::PodViewer::Parser qw(decode_entities);
-use Gnome2;
+use Gtk2::Pango;
+use Gtk2::Ex::Simple::List;
+use Gtk2::Ex::PodViewer 0.16;
+use Gtk2::Ex::PodViewer::Parser qw(decode_entities);
+use Gtk2::Ex::PrintDialog;
 use Locale::gettext;
+use Pod::Simple::Search;
 use POSIX qw(setlocale);
+use Storable;
+use URI::Escape;
+use HTML::Entities qw(encode_entities_numeric);
 use strict;
 
 ### set up global variables:
 my $NAME		= 'PodBrowser';
-my $VERSION		= '0.03';
+my $VERSION		= '0.10';
 my $PREFIX		= '@PREFIX@';
 my $GLADE_FILE		= (-d $PREFIX ? sprintf('%s/share/%s', $PREFIX, lc($NAME)) : $ENV{PWD}).sprintf('/%s.glade', lc($NAME));
 my $LOCALE_DIR		= (-d $PREFIX ? "PREFIX/share/locale" : $ENV{PWD}.'/locale');
-
-my $SEARCH_OFFSET	= 0;
 my $RCFILE		= sprintf('%s/.%src', $ENV{HOME}, lc($NAME));
+my $DOC_CACHE		= sprintf('%s/.%s_cache', $ENV{HOME}, lc($NAME));
+
+### instance specific ones:
+my $SEARCH_OFFSET	= 0;
 my $MAXIMIZED		= 0;
 my $FULLSCREEN		= 0;
 my $OPTIONS		= load_config();
 my @HISTORY		= split(/\|/, $OPTIONS->{history});
 my @BOOKMARKS		= split(/\|/, $OPTIONS->{bookmarks});
 my $BOOKMARK_ITEMS	= {};
+my $PATHS		= {};
+my $NO_REDRAW		= 0;
+my $MTIME		= 0;
 my @FORWARD;
 my @BACK;
 my $CURRENT_DOCUMENT;
 my $LAST_SEARCH_STR;
+my $LAST_FAILED_STR;
 
 ### set up l10n support:
 setlocale(LC_ALL, $ENV{LANG});
@@ -43,8 +57,8 @@ chomp(my $OPENER	= `which gnome-open 2> /dev/null`);
 my $APP			= Gtk2::GladeXML->new($GLADE_FILE);
 my $THEME		= get_an_icon_theme();
 my $TIPS		= Gtk2::Tooltips->new;
-my $IDX_PBF		= $APP->get_widget('main_window')->render_icon('gtk-jump-to', 'menu');
-my $PAGE_PBF		= Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon('gnome-mime-text', 16, 'force-svg')->get_filename)->scale_simple(16, 16, 'bilinear');
+my $IDX_PBF		= Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon('stock_bookmark', 16, 'force-svg')->get_filename)->scale_simple(16, 16, 'bilinear');
+my $PAGE_PBF		= Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon('stock_new-text', 16, 'force-svg')->get_filename)->scale_simple(16, 16, 'bilinear');
 my $FOLDER_PBF		= Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon('gnome-fs-directory', 16, 'force-svg')->get_filename)->scale_simple(16, 16, 'bilinear');
 my $NORMAL_CURSOR	= Gtk2::Gdk::Cursor->new('left_ptr');
 my $BUSY_CURSOR		= Gtk2::Gdk::Cursor->new('watch');
@@ -57,21 +71,61 @@ my %categories		= (
 );
 my $CATEGORY_PBFS	= {};
 
+my %TOOLBAR_STYLES = (
+	gettext('Icons')		=> 'icons',
+	gettext('Text')			=> 'text',
+	gettext('Both')			=> 'both',
+	gettext('Both Horizontal')	=> 'both-horiz',
+);
+
+$OPTIONS->{variable_font}	= (defined($OPTIONS->{variable_font})	? $OPTIONS->{variable_font}	: 'Sans 10');
+$OPTIONS->{fixed_font}		= (defined($OPTIONS->{fixed_font})	? $OPTIONS->{fixed_font}	: 'Monospace 10');
+$OPTIONS->{header_color}	= (defined($OPTIONS->{header_color})	? $OPTIONS->{header_color}	: '#404080');
+$OPTIONS->{preformat_color}	= (defined($OPTIONS->{preformat_color})	? $OPTIONS->{preformat_color}	: '#606060');
+$OPTIONS->{toolbar_style}	= (defined($OPTIONS->{toolbar_style})	? $OPTIONS->{toolbar_style}	: 'icons');
+
 ### start building the UI:
 
 $APP->signal_autoconnect_from_package(__PACKAGE__);
 $APP->get_widget('location')->disable_activate;
 $APP->get_widget('open_dialog_location')->disable_activate;
 
-my $viewer = Gtk2::PodViewer->new;
+# this seems not to be applied from the glade, so force it:
+$APP->get_widget('location_toolitem')->set_expand(1);
+
+my $group = Gtk2::AccelGroup->new;
+$group->connect(ord(1),   ['mod1-mask'],    ['visible'], sub { $APP->get_widget('notebook')->set_current_page(0) });
+$group->connect(ord(2),   ['mod1-mask'],    ['visible'], sub { $APP->get_widget('notebook')->set_current_page(1) });
+$group->connect(ord('L'), ['control-mask'], ['visible'], sub { $APP->get_widget('location')->entry->grab_focus });
+
+$APP->get_widget('main_window')->add_accel_group($group);
+
+my $toolbar_style_combo = Gtk2::ComboBox->new_text;
+for (my $i = 0 ; $i < scalar(keys(%TOOLBAR_STYLES)) ; $i++) {
+	my $style = (sort(keys(%TOOLBAR_STYLES)))[$i];
+	$toolbar_style_combo->append_text($style);
+	$toolbar_style_combo->set_active($i) if ($TOOLBAR_STYLES{$style} eq $OPTIONS->{toolbar_style});
+}
+$APP->get_widget('prefs_table')->attach_defaults($toolbar_style_combo, 1, 2, 4, 5);
+$toolbar_style_combo->show;
+
+my $viewer = Gtk2::Ex::PodViewer->new;
+
+$viewer->set_cursor_visible(undef);
 $viewer->signal_connect('link_clicked' => \&link_clicked);
+$APP->get_widget('viewer_scrwin')->add($viewer);
+
 $viewer->signal_connect('link_enter', sub { set_status($_[1]) });
 $viewer->signal_connect('link_leave', sub { set_status('') });
-$APP->get_widget('viewer_scrwin')->add($viewer);
+
+setup_display();
+
 $viewer->show;
 
+-r $DOC_CACHE ? $viewer->set_db(retrieve($DOC_CACHE)) :  $viewer->_init_db;
+
 ### build a SimpleList from the glade widget for the document index:
-my $page_index	= Gtk2::SimpleList->new_from_treeview(
+my $page_index	= Gtk2::Ex::Simple::List->new_from_treeview(
 	$APP->get_widget('document_index'),
 	icon	=> 'pixbuf',
 	mark	=> 'text',
@@ -94,55 +148,31 @@ $APP->get_widget('index')->insert_column_with_attributes(
 	Gtk2::CellRendererPixbuf->new,
 	pixbuf => 0,
 );
+my $col = Gtk2::CellRendererText->new;
+$col->set('ellipsize-set' => 0, 'ellipsize' => 'end');
 $APP->get_widget('index')->insert_column_with_attributes(
 	1,
 	'document',
-	Gtk2::CellRendererText->new,
+	$col,
 	text => 1,
 );
 
-Glib::Timeout->add(500, sub {
-	$ITEMS = generate_index();
-	### populate the tree:
-	foreach my $category (sort keys %{$ITEMS}) {
-		if ($category ne '') {
-			my $parent_iter = $model->append(undef);
-			$model->set($parent_iter, 0, (defined($CATEGORY_PBFS->{$category}) ? $CATEGORY_PBFS->{$category} : $FOLDER_PBF));
-			$model->set($parent_iter, 1, $categories{$category});
-			foreach my $doc (sort keys %{$ITEMS->{$category}}) {
-				if ($doc ne '') {
-					my $iter = $model->append($parent_iter);
-					$model->set($iter, 0, $PAGE_PBF);
-					$model->set($iter, 1, $doc);
-				}
-			}
-		}
-	}
-	return undef;
-});
-
 $APP->get_widget('index')->get_selection->signal_connect('changed', \&index_changed);
 
 eval {
-	$APP->get_widget('main_window')->set_icon(
-		Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon(lc($NAME), 16, 'force-svg')->get_filename)
-	);
+	my $icon = Gtk2::Gdk::Pixbuf->new_from_file($THEME->lookup_icon(lc($NAME), 16, 'force-svg')->get_filename);
+	$APP->get_widget('main_window')->set_icon($icon);
+	$APP->get_widget('view_source_window')->set_icon($icon);
+	$APP->get_widget('bookmarks_dialog')->set_icon($icon);
+	$APP->get_widget('display_preferences_dialog')->set_icon($icon);
 };
 print STDERR $@;
 
-### if the program was run with an argument, load the argument as a document:
-if ($ARGV[0] ne '') {
-	Glib::Timeout->add(50, sub {
-		set_location($ARGV[0]);
-		return undef;
-	});
-}
-
-Glib::Timeout->add(50, \&timeout);
+Glib::Timeout->add(1000, \&watch_file);
 
 ### apply the user's previous preferences:
-$APP->get_widget('pane')->set_position($OPTIONS->{pane_position}	|| 200);
-$APP->get_widget('vpane')->set_position(defined($OPTIONS->{vpane_position}) ? $OPTIONS->{vpane_position} : 250);
+$APP->get_widget('notebook')->set_current_page(int($OPTIONS->{active_page}));
+$APP->get_widget('pane')->set_position($OPTIONS->{pane_position} || 200);
 $APP->get_widget('show_index')->set_active($OPTIONS->{show_index} == 0 && defined($OPTIONS->{show_index}) ? undef : 1);
 $APP->get_widget('location')->set_popdown_strings('', @HISTORY);
 $APP->get_widget('main_window')->maximize if ($OPTIONS->{maximized} == 1);
@@ -151,6 +181,9 @@ $APP->get_widget('main_window')->set_default_size(
 	($OPTIONS->{window_y} > 0 ? $OPTIONS->{window_y} : 600),
 );
 
+$OPTIONS->{watch} = (!defined($OPTIONS->{watch}) ? 1 : ($OPTIONS->{watch} == 0 ? 0 : 1));
+$APP->get_widget('toggle_watch')->set_active($OPTIONS->{watch} == 0 ? undef : 1);
+
 $APP->get_widget('main_window')->signal_connect('window-state-event', \&window_changed_state);
 
 ### load the bookmarks:
@@ -159,7 +192,7 @@ foreach my $bookmark (@BOOKMARKS) {
 }
 
 ### construct the bookmarks list:
-my $bookmarks_list = Gtk2::SimpleList->new_from_treeview(
+my $bookmarks_list = Gtk2::Ex::Simple::List->new_from_treeview(
 	$APP->get_widget('bookmarks_list'),
 	icon	=> 'pixbuf',
 	doc	=> 'text',
@@ -167,64 +200,58 @@ my $bookmarks_list = Gtk2::SimpleList->new_from_treeview(
 
 $APP->get_widget('main_window')->show;
 
+my $completion_store = Gtk2::ListStore->new('Glib::String');
+
+load_index();
+
+my $completion = Gtk2::EntryCompletion->new;
+$completion->set_model($completion_store);
+$completion->set_text_column(0);
+$completion->set_inline_completion(1);
+$completion->signal_connect('match-selected', sub {
+	my ($completion, $model, $iter) = @_;
+	my $value = $model->get_value($iter, 0);
+	set_location($value);
+});
+
+my $completion2 = Gtk2::EntryCompletion->new;
+$completion2->set_model($completion_store);
+$completion2->set_text_column(0);
+$completion2->set_inline_completion(1);
+
+$APP->get_widget('location')->entry->set_completion($completion);
+$APP->get_widget('open_dialog_location')->entry->set_completion($completion2);
+
+$APP->get_widget('view_source_text')->set_border_window_size('top', 6);
+$APP->get_widget('view_source_text')->set_border_window_size('bottom', 6);
+$APP->get_widget('view_source_text')->set_border_window_size('left', 6);
+$APP->get_widget('view_source_text')->set_border_window_size('right', 6);
+$APP->get_widget('view_source_text')->modify_bg('normal', Gtk2::Gdk::Color->new(65535, 65535, 65535));
+
+$APP->get_widget('back_button')->set_menu(Gtk2::Menu->new);
+$APP->get_widget('forward_button')->set_menu(Gtk2::Menu->new);
+$APP->get_widget('up_button')->set_menu(Gtk2::Menu->new);
+
+### if the program was run with an argument, load the argument as a document:
+set_location($ARGV[0]) if ($ARGV[0] ne '');
+
 Gtk2->main;
 
 exit;
 
-### this runs every 50ms, and keeps track of the state of the navigation buttons and tooltips:
-sub timeout {
-	### the back button:
-	if (scalar(@BACK) > 0 && !$APP->get_widget('back_button')->get('sensitive')) {
-		$APP->get_widget('back_button')->set_sensitive(1);
-		$TIPS->set_tip($APP->get_widget('back_button'), sprintf(gettext("Go back to '%s' (right-click for more)"), $BACK[scalar(@BACK) - 1]));
+sub location_entry_changed {
+	$APP->get_widget('go_button')->set_sensitive($APP->get_widget('location')->entry->get_text ne '');
+	return 1;
+}
+sub search_entry_changed {
+	$APP->get_widget('search_button')->set_sensitive($APP->get_widget('search_entry')->get_text ne '');
+	return 1;
+}
 
-	} elsif (scalar(@BACK) < 1 && $APP->get_widget('back_button')->get('sensitive')) {
-		$APP->get_widget('back_button')->set_sensitive(undef);
-		$TIPS->set_tip($APP->get_widget('back_button'), '');
-
-	}
-
-	### the forward button:
-	if (scalar(@FORWARD) > 0 && !$APP->get_widget('forward_button')->get('sensitive')) {
-		$APP->get_widget('forward_button')->set_sensitive(1);
-		$TIPS->set_tip($APP->get_widget('forward_button'), sprintf(gettext("Go forward to '%s' (right-click for more)"), $FORWARD[0]));
-
-	} elsif (scalar(@FORWARD) < 1 && $APP->get_widget('forward_button')->get('sensitive')) {
-		$APP->get_widget('forward_button')->set_sensitive(undef);
-		$TIPS->set_tip($APP->get_widget('forward_button'), '');
-	}
-
-	### the up button:
-	if ($APP->get_widget('location')->entry->get_text =~ /::/) {
-		$APP->get_widget('up_button')->set_sensitive(1);
-		my @parts = split(/::/, $APP->get_widget('location')->entry->get_text);
-		pop(@parts);
-		my $doc = join('::', @parts);
-		$TIPS->set_tip($APP->get_widget('up_button'), sprintf(gettext("Go up to '%s' (right-click for more)"), $doc));
-
-	} else {
-		$APP->get_widget('up_button')->set_sensitive(undef);
-		$TIPS->set_tip($APP->get_widget('up_button'), '');
-
-	}
-
-	### the go button:
-	if ($APP->get_widget('location')->entry->get_text ne '' && !$APP->get_widget('go_button')->get('sensitive')) {
-		$APP->get_widget('go_button')->set_sensitive(1);
-
-	} elsif ($APP->get_widget('location')->entry->get_text eq '' && $APP->get_widget('go_button')->get('sensitive')) {
-		$APP->get_widget('go_button')->set_sensitive(undef);
-
-	}
-
-	### the "add bookmark" item. turn off if location is blank or is already bookmarked:
-	if ($APP->get_widget('location')->entry->get_text ne '' && !defined($BOOKMARK_ITEMS->{$APP->get_widget('location')->entry->get_text}) && !$APP->get_widget('add_bookmark_item')->get('sensitive')) {
-		$APP->get_widget('add_bookmark_item')->set_sensitive(1);
-
-	} elsif (($APP->get_widget('location')->entry->get_text eq '' || defined($BOOKMARK_ITEMS->{$APP->get_widget('location')->entry->get_text})) && $APP->get_widget('add_bookmark_item')->get('sensitive')) {
-		$APP->get_widget('add_bookmark_item')->set_sensitive(undef);
-
-	}
+sub watch_file {
+	return 1 if ($OPTIONS->{watch} != 1);
+	my $file = $APP->get_widget('location')->entry->get_text;
+	go() if ((stat($file))[9] > $MTIME);
 	return 1;
 }
 
@@ -274,59 +301,73 @@ sub browse_button_clicked {
 ### shows/hides the left pane of the window:
 sub toggle_index {
 	if ($_[0]->get_active) {
-		$APP->get_widget('vpane')->show_all;
+		$APP->get_widget('notebook')->show_all;
 	} else {
-		$APP->get_widget('vpane')->hide_all;
+		$APP->get_widget('notebook')->hide_all;
 	}
 	$OPTIONS->{show_index} = ($_[0]->get_active ? 1 : 0);
 
 	return 1;
 }
 
+sub toggle_watch {
+	$OPTIONS->{watch} = $APP->get_widget('toggle_watch')->get_active ? 1 : 0;
+}
+
 sub about {
-	my $about = Gnome2::About->new(
-		$NAME,
-		$VERSION,
-		gettext("This program is free software; you can redistribute\nit and/or modify it under the same terms as Perl itself."),
-		gettext('A Perl Documentation Browser for GNOME'),
-		[
-			'Gavin Brown',
-			'Torsten Schoenfeld',
-			'Scott Arrington',
-			'Steven Robson',
-		],
-		undef,
-		undef,
-		$APP->get_widget('main_window')->get_icon,
+	Gtk2::AboutDialog->set_url_hook(\&open_url);
+	Gtk2->show_about_dialog(
+		$APP->get_widget('main_window'),
+		name		=> $NAME,
+		version		=> $VERSION,
+		comments	=> gettext('A Perl Documentation Browser for GNOME'),,
+		copyright	=> gettext('Copyright 2005 Gavin Brown.'),
+		website		=> sprintf('http://jodrell.net/projects/%s', lc($NAME)),
+		icon		=> $APP->get_widget('main_window')->get_icon,
+		logo		=> $APP->get_widget('main_window')->get_icon,
 	);
-	$about->set_icon($APP->get_widget('main_window')->get_icon);
-	$about->show_all;
 	return 1;
 }
 
+sub set_ui_busy {
+	$APP->get_widget('main_window')->window->set_cursor($BUSY_CURSOR);
+	$viewer->get_window('text')->set_cursor($BUSY_CURSOR);
+	Gtk2->main_iteration while (Gtk2->events_pending);
+}
+
+sub set_ui_waiting {
+	$APP->get_widget('main_window')->window->set_cursor($NORMAL_CURSOR);
+	$viewer->get_window('text')->set_cursor($NORMAL_CURSOR);
+	Gtk2->main_iteration while (Gtk2->events_pending);
+}
+
 ### this is used when the user requests a new document, via the location entry,
 ### the go button, the 'open document' dialog, the index or via a clicked link:
 sub go {
+	my $no_reselect = shift;
 	my $text = $APP->get_widget('location')->entry->get_text;
 
-	$APP->get_widget('main_window')->window->set_cursor($BUSY_CURSOR);
-	$viewer->get_window('text')->set_cursor($BUSY_CURSOR);
-	Gtk2->main_iteration while (Gtk2->events_pending);
+	set_ui_busy();
 
 	if (!$viewer->load($text)) {
-		$APP->get_widget('location')->entry->set_text($CURRENT_DOCUMENT);
-		pop(@BACK);
-		$APP->get_widget('main_window')->window->set_cursor($NORMAL_CURSOR);
-		$viewer->get_window('text')->set_cursor($NORMAL_CURSOR);
-		my $dialog = Gtk2::MessageDialog->new($APP->get_widget('main_window'), 'modal', 'error', 'ok', sprintf(gettext("Couldn't find a POD document for '%s'."), $text));
-		$dialog->signal_connect('response', sub { $dialog->destroy });
-		$dialog->run;
+		my @children = grep { /^$text\:\:/ } keys(%{$viewer->get_db});
+		if (scalar(@children) > 0) {
+			$viewer->load_string(generate_pod_index($text, @children));
+			set_ui_waiting();
+
+		} else {
+			$APP->get_widget('location')->entry->set_text($CURRENT_DOCUMENT);
+			pop(@BACK);
+			set_ui_waiting();
+			show_fail_dialog($text);
+
+		}
 
 	} else {
+		$MTIME = time();
 		$CURRENT_DOCUMENT = $text;
 		$APP->get_widget('main_window')->set_title(sprintf(gettext('%s - Pod Browser'), $text));
-		$APP->get_widget('main_window')->window->set_cursor($NORMAL_CURSOR);
-		$viewer->get_window('text')->set_cursor($NORMAL_CURSOR);
+		set_ui_waiting();
 
 		### populate the index:
 		@{$page_index->{data}} = ();
@@ -345,28 +386,79 @@ sub go {
 		$APP->get_widget('location')->set_popdown_strings(@HISTORY);
 		$LAST_SEARCH_STR = '';
 
-		### this goes through the site index, looking to see if the requested document is present, and
-		### selects it if found:
-		my $found = 0;
-		for (my $i = 0 ; $i < scalar(keys(%categories)) ; $i++) {
-			my $category = (grep { $_ ne '' } sort(keys(%categories)))[$i];
-			for (my $j = 0 ; $j < scalar(keys(%{$ITEMS->{$category}})) ; $j++) {
-				my $doc = (grep { $_ ne '' } sort(keys(%{$ITEMS->{$category}})))[$j];
-				if ($doc eq $text) {
-					$found++;
-					$APP->get_widget('index')->expand_row(Gtk2::TreePath->new_from_indices($i), undef);
+		if (defined($PATHS->{$text}) && $no_reselect != 1) {
+			$APP->get_widget('index')->expand_row($PATHS->{$text}, 1);
+			$APP->get_widget('index')->scroll_to_cell($PATHS->{$text}, $APP->get_widget('index')->get_column(0), 1, 0.5, 0);
+			$NO_REDRAW = 1;
+			$APP->get_widget('index')->get_selection->select_path($PATHS->{$text});
+			$NO_REDRAW = 0;
+		}
+
+		### the back button:
+		if (scalar(@BACK) > 0) {
+			$APP->get_widget('back_button')->set_sensitive(1);
+			$APP->get_widget('back_button')->set_tooltip($TIPS, sprintf(gettext("Go back to '%s'"), $BACK[-1]), 1);
+
+		} elsif (scalar(@BACK) < 1 && $APP->get_widget('back_button')->get('sensitive')) {
+			$APP->get_widget('back_button')->set_sensitive(undef);
 
-					unless ($APP->get_widget('index')->has_focus) {
-						$APP->get_widget('index')->get_selection->select_path(Gtk2::TreePath->new_from_indices($i, $j));
-						$APP->get_widget('index')->scroll_to_cell(Gtk2::TreePath->new_from_indices($i, ($j <= 4 ? 0 : $j - 4)), undef, 1, 0, 0);
-					}
-				}
-			}
 		}
-		if ($found == 0) {
-			$APP->get_widget('index')->collapse_all;
-			$APP->get_widget('index')->get_selection->unselect_all;
+
+		### the forward button:
+		if (scalar(@FORWARD) > 0) {
+			$APP->get_widget('forward_button')->set_sensitive(1);
+			$APP->get_widget('forward_button')->set_tooltip($TIPS, sprintf(gettext("Go forward to '%s'"), $FORWARD[0]), 1);
+
+		} elsif (scalar(@FORWARD) < 1) {
+			$APP->get_widget('forward_button')->set_sensitive(undef);
+
 		}
+
+		### the up button:
+		if ($APP->get_widget('location')->entry->get_text =~ /::/) {
+			$APP->get_widget('up_button')->set_sensitive(1);
+			my @parts = split(/::/, $APP->get_widget('location')->entry->get_text);
+			pop(@parts);
+			my $doc = join('::', @parts);
+			$APP->get_widget('up_button')->set_tooltip($TIPS, sprintf(gettext("Go up to '%s'"), $doc), 1);
+
+		} else {
+			$APP->get_widget('up_button')->set_sensitive(undef);
+
+		}
+
+		### the "add bookmark" item. turn off if location is blank or is already bookmarked:
+		$APP->get_widget('add_bookmark_item')->set_sensitive($APP->get_widget('location')->entry->get_text ne '' && !defined($BOOKMARK_ITEMS->{$APP->get_widget('location')->entry->get_text}));
+
+		return 1;
+
+	}
+
+	return 1;
+}
+
+sub show_fail_dialog {
+	my $text = shift;
+	$LAST_FAILED_STR = $text;
+	$APP->get_widget('load_failed_dialog_title_label')->set_markup(sprintf('<span size="large" weight="bold">%s</span>',  encode_entities_numeric(sprintf(gettext("Couldn't find a POD document for '%s'."), $text))));
+	$APP->get_widget('load_failed_dialog')->show_all;
+}
+
+sub close_load_failed_dialog {
+	$APP->get_widget('load_failed_dialog')->hide;
+	return 1;
+}
+
+sub load_failed_dialog_response {
+	close_load_failed_dialog();
+
+	if ($_[1] == 49) {
+		open_url(sprintf('http://search.cpan.org/search?query=%s', uri_escape($LAST_FAILED_STR)));
+
+	} elsif ($_[1] == 99) {
+		reload_index();
+		set_location($LAST_FAILED_STR);
+
 	}
 
 	return 1;
@@ -382,11 +474,12 @@ sub close_program {
 		($OPTIONS->{window_x}, $OPTIONS->{window_y}) = $APP->get_widget('main_window')->get_size if ($FULLSCREEN == 0);
 	}
 
+	$OPTIONS->{active_page} = $APP->get_widget('notebook')->get_current_page;
 	$OPTIONS->{pane_position} = $APP->get_widget('pane')->get_position;
-	$OPTIONS->{vpane_position} = $APP->get_widget('vpane')->get_position;
 	$OPTIONS->{history} = join('|', splice(@HISTORY, 0, 20));
 	$OPTIONS->{bookmarks} = join('|', @BOOKMARKS);
 	save_config();
+	save_cache();
 	exit 0;
 }
 
@@ -417,6 +510,10 @@ sub save_config {
 	return undef;
 }
 
+sub save_cache {
+	store($viewer->get_db, $DOC_CACHE);
+}
+
 sub link_clicked {
 	my (undef, $text) = @_;
 	$text =~ s/\"$//g;
@@ -462,17 +559,12 @@ sub link_clicked {
 	} else {
 		# link referred to another pod document:
 		set_location($text);
+
 	}
 	return 1;
 
 }
 
-sub set_status {
-	my $str = shift;
-	$APP->get_widget('status_bar')->push($APP->get_widget('status_bar')->get_context_id($str), $str);
-	return 1;
-}
-
 sub set_location {
 	my $locn = shift;
 	if ($APP->get_widget('location')->entry->get_text ne '') {
@@ -483,24 +575,6 @@ sub set_location {
 	go();
 }
 
-### if it's a left-click, go back, if it's a right-click, run the popup (this
-### works the same for the other two navigation buttons:
-sub back_button_handler {
-	if ($_[1]->button == 1) {
-		go_back();
-	} elsif ($_[1]->button == 3) {
-		back_button_popup();
-	}
-	return 1;
-}
-
-### this is called by the function above, and by the 'popup_menu' signal on the
-### button:
-sub back_button_popup() {
-	popup_menu(3, reverse(@BACK));
-	return 1;
-}
-
 sub go_back {
 	unshift(@FORWARD, $APP->get_widget('location')->entry->get_text);
 	my $locn = pop(@BACK);
@@ -508,20 +582,6 @@ sub go_back {
 	go();
 }
 
-sub forward_button_handler {
-	if ($_[1]->button == 1) {
-		go_forward();
-	} elsif ($_[1]->button == 3) {
-		forward_button_popup();
-	}
-	return 1;
-}
-
-sub forward_button_popup() {
-	popup_menu(3, @FORWARD);
-	return 1;
-}
-
 sub go_forward {
 	push(@BACK, $APP->get_widget('location')->entry->get_text);
 	my $locn = shift(@FORWARD);
@@ -529,28 +589,6 @@ sub go_forward {
 	go();
 }
 
-sub up_button_handler {
-	if ($_[1]->button == 1) {
-		go_up();
-	} elsif ($_[1]->button == 3) {
-		up_button_popup();
-	}
-	return 1;
-}
-
-sub up_button_popup() {
-	my @items;
-	my @parts;
-	my $doc = $APP->get_widget('location')->entry->get_text;
-	foreach my $part (split(/::/, $doc)) {
-		push(@parts, $part);
-		my $this_doc = join('::', @parts);
-		push(@items, $this_doc) unless ($this_doc eq $doc);
-	}
-	popup_menu(3, @items);
-	return 1;
-}
-
 sub go_up {
 	my @parts = split(/::/, $APP->get_widget('location')->entry->get_text);
 	pop(@parts);
@@ -560,6 +598,29 @@ sub go_up {
 	go();
 }
 
+sub show_back_button_menu {
+	create_popup_menu($APP->get_widget('back_button')->get_menu, @BACK);
+	return 1;
+}
+
+sub show_forward_button_menu {
+	create_popup_menu($APP->get_widget('forward_button')->get_menu, @FORWARD);
+	return 1;
+}
+
+sub show_up_button_menu {
+	my @items = ();
+	my @parts = ();
+	my $doc = $APP->get_widget('location')->entry->get_text;
+	foreach my $part (split(/::/, $doc)) {
+		push(@parts, $part);
+		my $this_doc = join('::', @parts);
+		push(@items, $this_doc) unless ($this_doc eq $doc);
+	}
+	create_popup_menu($APP->get_widget('up_button')->get_menu, @items);
+	return 1;
+}
+
 sub user_set_location {
 	return undef if ($APP->get_widget('location')->entry->get_text eq '');
 	push(@BACK, $CURRENT_DOCUMENT) if ($CURRENT_DOCUMENT ne '' && $CURRENT_DOCUMENT ne $APP->get_widget('location')->entry->get_text);
@@ -574,54 +635,51 @@ sub search {
 
 	return undef if ($str eq '');
 
-	set_status('Searching...');
-
-	$APP->get_widget('main_window')->window->set_cursor($BUSY_CURSOR);
-	$viewer->get_window('text')->set_cursor($BUSY_CURSOR);
+	set_ui_busy();
 
 	my $doc = $viewer->get_buffer->get_text(
 		$viewer->get_buffer->get_start_iter,
 		$viewer->get_buffer->get_end_iter,
-		1
+		undef,
 	);
 
-	$str = quotemeta($str);
 	$APP->get_widget('search_entry')->set_sensitive(0);
 
 	$SEARCH_OFFSET = 0 if ($str ne $LAST_SEARCH_STR);
 	$LAST_SEARCH_STR = $str;
 
-	for ($SEARCH_OFFSET ; $SEARCH_OFFSET < length($doc) ; $SEARCH_OFFSET++) {
-		Gtk2->main_iteration while (Gtk2->events_pending);
-		if (substr($doc, $SEARCH_OFFSET) =~ /^$str/i) {
-			my $iter = $viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET);
-			$viewer->scroll_to_iter($iter, undef, 1, 0, 0);
-			$APP->get_widget('search_entry')->set_sensitive(1);
-			$APP->get_widget('search_entry')->grab_focus();
-			$viewer->get_buffer->move_mark(
-				$viewer->get_buffer->get_mark('insert'), 
-				$viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET)
-			);
-			$viewer->get_buffer->move_mark(
-				$viewer->get_buffer->get_mark('selection_bound'), 
-				$viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET + length($str))
-			);
+	my $chunk = substr($doc, $SEARCH_OFFSET);
 
-			set_status('');
-			$APP->get_widget('main_window')->window->set_cursor($NORMAL_CURSOR);
-			$SEARCH_OFFSET += length($str);
-			return 1;
-		}
+	while ($chunk =~ /($str)/ig) {
+		Gtk2->main_iteration while (Gtk2->events_pending);
+
+		$SEARCH_OFFSET += pos($chunk);
+
+		my $iter = $viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET);
+		$viewer->scroll_to_iter($iter, undef, 1, 0, 0);
+
+		$APP->get_widget('search_entry')->set_sensitive(1);
+		$APP->get_widget('search_entry')->grab_focus();
+
+		$viewer->get_buffer->move_mark(
+			$viewer->get_buffer->get_mark('insert'), 
+			$viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET - length($1)),
+		);
+		$viewer->get_buffer->move_mark(
+			$viewer->get_buffer->get_mark('selection_bound'), 
+			$viewer->get_buffer->get_iter_at_offset($SEARCH_OFFSET),
+		);
+
+		set_ui_waiting();
+
+		return 1;
 	}
+
 	$APP->get_widget('search_entry')->set_sensitive(1);
 	$APP->get_widget('search_entry')->grab_focus();
 
-	set_status('');
-
 	$SEARCH_OFFSET = 0;
-
-	$APP->get_widget('main_window')->window->set_cursor($NORMAL_CURSOR);
-	$viewer->get_window('text')->set_cursor($NORMAL_CURSOR);
+	set_ui_waiting();
 
 	my $dialog = Gtk2::MessageDialog->new($APP->get_widget('main_window'), 'modal', 'info', 'ok', sprintf(gettext("The string '%s' was not found."), $str));
 	$dialog->signal_connect('response', sub { $dialog->destroy });
@@ -670,10 +728,11 @@ sub search_dialog_response {
 }
 
 sub open_url {
-	my $url = shift;
+	my $url = (ref($_[0]) eq 'Gtk2::AboutDialog' ? $_[1] : $_[0]);
 
 	if (!-x $OPENER) {
-		my $dialog = Gtk2::MessageDialog->new($APP->get_widget('main_window'), 'modal', 'info', 'ok', gettext("Cannot find the gnome-open program to launch this URL was not found."));
+		my $dialog = Gtk2::MessageDialog->new($APP->get_widget('main_window'), 'modal', 'info', 'ok', gettext('Error opening URL'));
+		$dialog->format_secondary_text("The 'gnome-open' program could not be found.");
 		$dialog->signal_connect('response', sub { $dialog->destroy });
 		$dialog->show_all;
 		return undef;
@@ -689,50 +748,30 @@ sub open_url {
 ### documents, and creates a hash of hashes that can be used to build a sitewide index:
 sub generate_index {
 	my %PATHS;
-
-	# doing a reverse sort means that later versions of Perl are preferred over newer versions:
-	foreach my $dir (reverse sort @INC) {
-		if (-r "$dir/pod/perltoc.pod" && $PATHS{perltoc} eq '') {
-			$PATHS{perltoc} = "$dir/pod/perltoc.pod";
-		}
-
-		if (-r "$dir/pod/perlfunc.pod" && $PATHS{perlfunc} eq '') {
-			$PATHS{perlfunc} = "$dir/pod/perlfunc.pod";
-		}
-
-		if (-r "$dir/perllocal.pod" && $PATHS{perllocal} eq '') {
-			$PATHS{perllocal} = "$dir/perllocal.pod";
-		}
-	}
-
 	my $ITEMS = {};
 	my $category;
 
-	if (-r $PATHS{perltoc}) {
-		if (!open(PERLTOC, $PATHS{perltoc})) {
-			print STDERR "$PATHS{perltoc}: $!\n";
+	my @docs = sort(keys(%{$viewer->get_db}));
 
-		} else {
-			while (<PERLTOC>) {
-				if (/BASIC DOCUMENTATION/) {
-					$category = 'pods';
+	foreach my $doc (@docs) {
+		if ($doc =~ /^perl/) {
+			$category = 'pods';
 
-				} elsif (/PRAGMA DOCUMENTATION/) {
-					$category = 'pragma';
+		} elsif ($doc =~ /^[a-z0-9]+$/) {
+			$category = 'pragma';
 
-				} elsif (/MODULE DOCUMENTATION/) {
-					$category = 'modules';
+		} elsif ($doc =~ /^[A-Z0-9]/ || $doc =~ /::/) {
+			$category = 'modules';
 
-				} elsif (/AUXILIARY DOCUMENTATION/) {
-					$category = 'aux';
+		}
+		$ITEMS->{$category}->{$doc}++;
+	}
 
-				} elsif (/^=head2/) {
-					my (undef, $doc, undef) = split(' ', $_, 3);
-					$doc =~ s/[^A-Za-z0-9\:\_]//g;
-					$ITEMS->{$category}->{$doc}++;
-				}
-			}
-			close(PERLTOC);
+	# doing a reverse sort means that later versions of Perl are preferred over newer versions:
+	foreach my $dir (reverse sort @INC) {
+		if (-r "$dir/pod/perlfunc.pod" && $PATHS{perlfunc} eq '') {
+			$PATHS{perlfunc} = "$dir/pod/perlfunc.pod";
+			last;
 		}
 	}
 
@@ -755,23 +794,41 @@ sub generate_index {
 		}
 	}
 
-	if (-r $PATHS{perllocal}) {
-		if (!open(PERLLOCAL, $PATHS{perllocal})) {
-			print STDERR "$PATHS{perllocal}: $!\n";
+	return $ITEMS;
+}
 
-		} else {
-			while (<PERLLOCAL>) {
-				if (/^=head2 .+? L<([A-Za-z0-9\:]+)\|/) {
-					my $module = $1;
-					$module =~ s/[^A-Za-z0-9\:\_]//g;
-					$ITEMS->{modules}->{$1}++;
+sub load_index {
+	set_ui_busy();
+	$model->clear;
+	$ITEMS = generate_index();
+	### populate the tree:
+	foreach my $category (sort keys %{$ITEMS}) {
+		Gtk2->main_iteration while (Gtk2->events_pending);
+		if ($category ne '') {
+			my $parent_iter = $model->append(undef);
+			$model->set($parent_iter, 0, (defined($CATEGORY_PBFS->{$category}) ? $CATEGORY_PBFS->{$category} : $FOLDER_PBF));
+			$model->set($parent_iter, 1, $categories{$category});
+			foreach my $doc (sort keys %{$ITEMS->{$category}}) {
+				Gtk2->main_iteration while (Gtk2->events_pending);
+				if ($doc ne '') {
+					my $iter = $model->append($parent_iter);
+					$PATHS->{$doc} = $model->get_path($iter);
+					$model->set($iter, 0, $PAGE_PBF);
+					$model->set($iter, 1, $doc);
 				}
 			}
-			close(PERLLOCAL);
 		}
 	}
 
-	return $ITEMS;
+	$completion_store->clear;
+	my @items = sort(keys(%{$viewer->get_db}), keys(%{$ITEMS->{funcs}}));
+	foreach my $doc (@items) {
+		$completion_store->set($completion_store->append, 0, $doc);
+	}
+
+	set_ui_waiting();
+
+	return undef;
 }
 
 sub section_reformat {
@@ -814,7 +871,11 @@ sub window_changed_state {
 ### this is called when the user clicks on an item in the site index:
 my $index_changed_timeout;
 sub index_changed {
+
+	return 1 if ($NO_REDRAW == 1);
+
 	my ($path) = $APP->get_widget('index')->get_selection->get_selected_rows;
+
 	if (defined($path)) {
 		my $path = $path->to_string;
 		my $iter = $APP->get_widget('index')->get_model->get_iter_from_string($path);
@@ -829,7 +890,7 @@ sub index_changed {
 			Glib::Source->remove($index_changed_timeout) if ($index_changed_timeout);
 			$index_changed_timeout = Glib::Timeout->add(200, sub {
 				$APP->get_widget('location')->entry->set_text($value);
-				go();
+				go(1);
 				$index_changed_timeout = 0;
 				0; # don't run again
 			});
@@ -923,6 +984,7 @@ sub add_bookmark {
 	my $bookmark = $APP->get_widget('location')->entry->get_text;
 	add_bookmark_item($bookmark);
 	push(@BOOKMARKS, $bookmark);
+	$APP->get_widget('add_bookmark_item')->set_sensitive($APP->get_widget('location')->entry->get_text ne '' && !defined($BOOKMARK_ITEMS->{$APP->get_widget('location')->entry->get_text}));
 	return 1;
 }
 
@@ -956,19 +1018,157 @@ sub document_menu_item {
 }
 
 ### create a popup menu for a list of documents:
-sub popup_menu {
-	my ($button, @docs) = @_;
-	my $menu = Gtk2::Menu->new;
+sub create_popup_menu {
+	my ($menu, @docs) = @_;
+	foreach my $child ($menu->get_children) {
+		$menu->remove($child);
+		$child->destroy;
+	}
 	foreach my $doc (@docs) {
 		my $item = document_menu_item($doc);
-		$item->show_all;
 		$menu->append($item);
 	}
 	$menu->show_all;
-	$menu->popup(undef, undef, undef, undef, $button, undef);
 	return 1;
 }
 
+sub reload_index {
+	set_ui_busy();
+	$APP->get_widget('main_window')->set_sensitive(undef);
+	Gtk2->main_iteration while(Gtk2->events_pending);
+	$viewer->reinitialize_db;
+	save_cache();
+	$APP->get_widget('main_window')->set_sensitive(1);
+	set_ui_waiting();
+}
+
+sub show_help {
+	set_location($0);
+}
+
+sub view_source {
+	$APP->get_widget('view_source_text')->get_buffer->set_text($viewer->parser->source);
+	$APP->get_widget('view_source_window')->show_all;
+}
+
+sub view_source_window_hide {
+	$APP->get_widget('view_source_window')->hide;
+	return 1;
+}
+
+sub show_display_preferences_dialog {
+	$APP->get_widget('variable_font_chooser')->set_font_name($OPTIONS->{variable_font});
+	$APP->get_widget('fixed_font_chooser')->set_font_name($OPTIONS->{fixed_font});
+
+	$APP->get_widget('header_color_chooser')->set_color(Gtk2::Gdk::Color->parse($OPTIONS->{header_color}));
+	$APP->get_widget('monospace_color_chooser')->set_color(Gtk2::Gdk::Color->parse($OPTIONS->{preformat_color}));
+
+	$APP->get_widget('display_preferences_dialog')->show;
+	return 1;
+}
+
+sub close_display_preferences_dialog {
+	$APP->get_widget('display_preferences_dialog')->hide;
+	return 1;
+}
+
+sub display_preferences_dialog_response {
+	if ($_[1] eq 'ok') {
+		$OPTIONS->{variable_font} = $APP->get_widget('variable_font_chooser')->get_font_name;
+		$OPTIONS->{fixed_font} = $APP->get_widget('fixed_font_chooser')->get_font_name;
+
+		$OPTIONS->{toolbar_style} = $TOOLBAR_STYLES{$toolbar_style_combo->get_active_text};
+
+		$OPTIONS->{header_color} = sprintf('#%02X%02X%02X',
+			$APP->get_widget('header_color_chooser')->get_color->red / 256,
+			$APP->get_widget('header_color_chooser')->get_color->green / 256,
+			$APP->get_widget('header_color_chooser')->get_color->blue / 256,
+		);
+
+		$OPTIONS->{preformat_color} = sprintf('#%02X%02X%02X',
+			$APP->get_widget('monospace_color_chooser')->get_color->red / 256,
+			$APP->get_widget('monospace_color_chooser')->get_color->green / 256,
+			$APP->get_widget('monospace_color_chooser')->get_color->blue / 256,
+		);
+
+		setup_display();
+	}
+	$APP->get_widget('display_preferences_dialog')->hide;
+	return 1;
+}
+
+sub setup_display {
+	$viewer->modify_font(Gtk2::Pango::FontDescription->from_string($OPTIONS->{variable_font}));
+	$APP->get_widget('view_source_text')->modify_font(Gtk2::Pango::FontDescription->from_string($OPTIONS->{fixed_font}));
+	$APP->get_widget('toolbar')->set_style($OPTIONS->{toolbar_style});
+	($OPTIONS->{toolbar_style} eq 'icons' ? $APP->get_widget('location_label')->hide : $APP->get_widget('location_label')->show);
+	$viewer->get_buffer->get_tag_table->lookup('monospace')->set('font' => $OPTIONS->{fixed_font});
+	$viewer->get_buffer->get_tag_table->lookup('monospace')->set('foreground' => $OPTIONS->{preformat_color});
+	$viewer->get_buffer->get_tag_table->lookup('typewriter')->set('font' => $OPTIONS->{fixed_font});
+	$viewer->get_buffer->get_tag_table->lookup('head1')->set('foreground' => $OPTIONS->{header_color});
+	$viewer->get_buffer->get_tag_table->lookup('head2')->set('foreground' => $OPTIONS->{header_color});
+	$viewer->get_buffer->get_tag_table->lookup('head3')->set('foreground' => $OPTIONS->{header_color});
+	$viewer->get_buffer->get_tag_table->lookup('head4')->set('foreground' => $OPTIONS->{header_color});
+}
+
+sub show_print_dialog {
+	my $source = $viewer->parser->source;
+
+	$source = "=over\n\n".$source."\n\n=back\n" if ($source =~ /^=item/);
+	$source = "=pod\n\n".$source."\n\n=cut\n" if ($source !~ /^=pod/m);
+
+	$APP->get_widget('main_window')->set_sensitive(undef);
+	set_ui_busy();
+
+	my $tempdir = tempdir(CLEANUP => 1);
+	my ($fh, $file) = tempfile(DIR=>$tempdir);
+	print $fh $viewer->parser->source;
+	close($fh);
+
+	my $title = $APP->get_widget('location')->entry->get_text;
+
+	my $cwd = $ENV{PWD};
+	my $cmd = sprintf('pod2html --noindex --infile "%s" --title "%s" 2>/dev/null | %s/html2ps-%s 2>/dev/null', $file, $title, dirname($0), lc($NAME));
+	chdir($tempdir);
+	if (!open(CMD, "$cmd|")) {
+		print STDERR $!;
+
+		set_ui_waiting();
+		$APP->get_widget('main_window')->set_sensitive(1);
+
+	} else {
+		my $ps = '';
+		while (<CMD>) {
+			$ps .= $_;
+		}
+		close(CMD);
+		unlink($file);
+
+		my $dialog = Gtk2::Ex::PrintDialog->new;
+		$dialog->set_data($ps);
+
+		set_ui_waiting();
+		$APP->get_widget('main_window')->set_sensitive(1);
+
+		$dialog->run;
+
+	}
+	chdir($ENV{PWD});
+}
+
+sub generate_pod_index {
+	my ($name, @docs) = @_;
+	my $pod = sprintf("=pod\n\n=head1 %s\n\n=over\n\n%s\n\n", $name, sprintf(gettext('Documents under %s:'), $name));
+	foreach my $doc (sort(@docs)) {
+		$pod .= sprintf("=item L<%s>\n\n", $doc);
+	}
+	return $pod . "=back\n\n=cut\n\n";
+}
+
+sub set_status {
+	$APP->get_widget('status')->push($APP->get_widget('status')->get_context_id('Default'), shift);
+}
+
 __END__
 
 =pod
@@ -978,7 +1178,7 @@ __END__
 podbrowser - a Perl documentation browser for GNOME
 
 PodBrowser is a more feature-complete version of podviewer, which comes with
-Gtk2::PodViewer.
+Gtk2::Ex::PodViewer.
 
 =head1 SYNTAX
 
@@ -994,8 +1194,36 @@ default and user-installed modules.
 
 F<location> If an argument is specified this argument is loaded as location.
 
+=head1 PREREQUISITES
+
+In addition to a number of modules bundles with recent Perl releases,
+PodBrowser needs the following:
+
+=over
+
+=item L<Gtk2> and the C<gtk2> library (C<gtk2> E<gt>= 2.8.0 is required)
+
+=item The C<gnome-icon-theme> package (version E<gt>= 2.10.0 is required)
+
+=item L<Gtk2::GladeXML> and the C<libglade> library
+
+=item L<Locale::gettext> and the C<gettext> library
+
+=item L<Gtk2::Ex::PodViewer>
+
+=item L<Gtk2::Ex::PrintDialog>
+
+=item L<Pod::Simple::Search>
+
+=item L<URI::Escape>
+
+=back
+
 =head1 AUTHOR
 
-Gavin Brown E<lt>L<gavin.brown@uk.com>E<gt>. Original manpage by Florian Ragwitz E<lt>florian@mookooh.orgE<gt>.
+(C) Gavin Brown. Original manpage by Florian Ragwitz.
+
+The C<html2ps-podbrowser> script is a copy of the original C<html2ps> script which
+is (C) Jan KE<auml>rrman.
 
 =cut
